from enum import Enum
from typing import Any
from typing import Generic
from typing import Optional
from typing import Type
from typing import TYPE_CHECKING
from typing import TypeVar

import pynamodb.constants
from pynamodb.attributes import Attribute

T = TypeVar('T', bound=Enum)
_fail: Any = object()


class UnicodeEnumAttribute(Attribute, Generic[T]):
    """
    Stores string enumerations (Enums whose values are strings) as DynamoDB strings.

    >>> from enum import Enum
    >>>
    >>> from pynamodb.models import Model
    >>>
    >>> class ShakeFlavor(Enum):
    >>>   VANILLA = 'vanilla'
    >>>   CHOCOLATE = 'chocolate'
    >>>   COOKIES = 'cookies'
    >>>   MINT = 'mint'
    >>>
    >>> class Shake(Model):
    >>>   flavor = UnicodeEnumAttribute(ShakeFlavor)
    """
    attr_type = pynamodb.constants.STRING

    def __init__(self, enum_type: Type[T], unknown_value: Optional[T] = _fail, **kwargs: Any) -> None:
        """
        :param enum_type: The type of the enum
        """
        super().__init__(**kwargs)
        self.enum_type = enum_type
        self.unknown_value = unknown_value
        if not all(isinstance(e.value, str) for e in self.enum_type):
            raise TypeError(f"Enumeration '{self.enum_type}' values must be all strings")

    def deserialize(self, value: str) -> Optional[T]:
        try:
            return self.enum_type(value)
        except ValueError:
            if self.unknown_value is _fail:
                raise
            return self.unknown_value

    def serialize(self, value: T) -> str:
        if not isinstance(value, self.enum_type):
            raise TypeError(f"value has invalid type '{type(value)}'; expected '{self.enum_type}'")
        return value.value

    if TYPE_CHECKING:
        def __get__(self, instance: Any, owner: Any) -> T:
            ...



import json
from datetime import date
from typing import Any
from typing import TYPE_CHECKING

from pynamodb.attributes import Attribute

from pynamodb_attributes import IntegerAttribute


class IntegerDateAttribute(Attribute):
    """Represents a date as an integer (e.g. 2015_12_31 for December 31st, 2015)."""
    attr_type = IntegerAttribute.attr_type

    def serialize(self, value: date) -> str:
        return json.dumps(value.year * 1_00_00 + value.month * 1_00 + value.day)

    def deserialize(self, value: str) -> date:
        n = json.loads(value)
        return date(n // 1_00_00, n // 1_00 % 1_00, n % 1_00)

    if TYPE_CHECKING:
        def __get__(self, instance: Any, owner: Any) -> date:
            ...



from .integer import IntegerAttribute
from .integer_date import IntegerDateAttribute
from .unicode_delimited_tuple import UnicodeDelimitedTupleAttribute
from .unicode_enum import UnicodeEnumAttribute

__all__ = [
    'IntegerAttribute',
    'IntegerDateAttribute',
    'UnicodeDelimitedTupleAttribute',
    'UnicodeEnumAttribute',
]



from typing import Any
from typing import Generic
from typing import Tuple
from typing import Type
from typing import TYPE_CHECKING
from typing import TypeVar

import pynamodb.constants
from pynamodb.attributes import Attribute

T = TypeVar('T', bound=tuple)
_DEFAULT_FIELD_DELIMITER = '::'


class UnicodeDelimitedTupleAttribute(Attribute, Generic[T]):
    """
    Stores a tuple of strings as a string. The tuple's members will be joined with a delimiter.

    >>> from typing import NamedTuple
    >>>
    >>> from pynamodb.models import Model
    >>>
    >>> class LatLng(NamedTuple):
    >>>   lat: int
    >>>   lng: int
    >>>
    >>> class Employee(Model):
    >>>   location = UnicodeDelimitedTupleAttribute(LatLng)
    """

    attr_type = pynamodb.constants.STRING

    def __init__(self, tuple_type: Type[T], delimiter: str = _DEFAULT_FIELD_DELIMITER, **kwargs: Any) -> None:
        """
        :param tuple_type: The type of the tuple -- may be a named or plain tuple
        :param delimiter: The delimiter to separate the tuple elements
        """
        super().__init__(**kwargs)
        self.tuple_type: Type[Tuple] = tuple_type
        self.delimiter = delimiter

    def deserialize(self, value: str) -> T:
        fields = getattr(self.tuple_type, '_fields', None)
        field_types = getattr(self.tuple_type, '_field_types', None)
        if fields and field_types:
            values = value.split(self.delimiter, maxsplit=len(fields))
            return self.tuple_type(**{f: field_types[f](v) for f, v in zip(fields, values)})  # type: ignore
        else:
            return self.tuple_type(value.split(self.delimiter))  # type: ignore

    def serialize(self, value: T) -> str:
        if not isinstance(value, self.tuple_type):
            raise TypeError(f"value has invalid type '{type(value)}'; expected '{self.tuple_type}'")
        values = [e for e in value]
        while values and values[-1] is None:
            del values[-1]
        strings = [str(e) for e in values]
        if any(self.delimiter in s for s in strings):
            raise ValueError(f"Tuple elements may not contain delimiter '{self.delimiter}'")
        return self.delimiter.join(strings)

    if TYPE_CHECKING:
        def __get__(self, instance: Any, owner: Any) -> T:
            ...



from typing import Any
from typing import TYPE_CHECKING

from pynamodb.attributes import Attribute
from pynamodb.attributes import NumberAttribute


class IntegerAttribute(Attribute):
    """
    Unlike NumberAttribute, this attribute has its type hinted as 'int'.
    """
    attr_type = NumberAttribute.attr_type
    serialize = NumberAttribute.serialize
    deserialize = NumberAttribute.deserialize

    if TYPE_CHECKING:
        def __get__(self, instance: Any, owner: Any) -> int:
            ...



from setuptools import find_packages
from setuptools import setup

setup(
    name='pynamodb-attributes',
    version='0.1.1',
    description='Common attributes for PynamoDB',
    url='https://www.github.com/lyft/pynamodb-attributes',
    maintainer='Lyft',
    maintainer_email='ikonstantinov@lyft.com',
    packages=find_packages(exclude=['tests*']),
    dependency_links=[],
    install_requires=[
        "pynamodb",
    ],
    python_requires='>=3',
    package_data={'pynamodb_attributes': ['py.typed']},
)
